# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redditwarp',
 'redditwarp._cli',
 'redditwarp.auth',
 'redditwarp.core',
 'redditwarp.dto_models',
 'redditwarp.dto_models.widget',
 'redditwarp.http',
 'redditwarp.http.misc',
 'redditwarp.http.transport',
 'redditwarp.http.transport.carriers',
 'redditwarp.http.transporters',
 'redditwarp.http.util',
 'redditwarp.internal_api',
 'redditwarp.internal_api.auth',
 'redditwarp.internal_api.core',
 'redditwarp.internal_api.siteprocs',
 'redditwarp.iterators',
 'redditwarp.model_loaders',
 'redditwarp.models',
 'redditwarp.models.widget',
 'redditwarp.pagination',
 'redditwarp.pagination.paginators',
 'redditwarp.pagination.paginators.listing',
 'redditwarp.pagination.paginators.listing.mixins',
 'redditwarp.pagination.paginators.moderation',
 'redditwarp.pagination.paginators.user',
 'redditwarp.pushshift',
 'redditwarp.pushshift.core',
 'redditwarp.pushshift.models',
 'redditwarp.pushshift.paginators',
 'redditwarp.pushshift.siteprocs',
 'redditwarp.siteprocs',
 'redditwarp.siteprocs.account',
 'redditwarp.siteprocs.collection',
 'redditwarp.siteprocs.comment',
 'redditwarp.siteprocs.comment_tree',
 'redditwarp.siteprocs.custom_feed',
 'redditwarp.siteprocs.draft',
 'redditwarp.siteprocs.flair',
 'redditwarp.siteprocs.flair_emoji',
 'redditwarp.siteprocs.front',
 'redditwarp.siteprocs.legacy_reddit_subreddit_style',
 'redditwarp.siteprocs.live_thread',
 'redditwarp.siteprocs.message',
 'redditwarp.siteprocs.misc',
 'redditwarp.siteprocs.moderation',
 'redditwarp.siteprocs.modmail',
 'redditwarp.siteprocs.redesign_reddit_subreddit_style',
 'redditwarp.siteprocs.submission',
 'redditwarp.siteprocs.subreddit',
 'redditwarp.siteprocs.user',
 'redditwarp.siteprocs.widget',
 'redditwarp.siteprocs.wiki',
 'redditwarp.streaming',
 'redditwarp.streaming.carriers',
 'redditwarp.streaming.carriers.main_async',
 'redditwarp.streaming.carriers.main_async.makers',
 'redditwarp.streaming.carriers.main_sync',
 'redditwarp.streaming.carriers.main_sync.makers',
 'redditwarp.streaming.makers',
 'redditwarp.streaming.util',
 'redditwarp.util',
 'redditwarp.websocket',
 'redditwarp.websocket.transport',
 'redditwarp.websocket.transport.carriers',
 'redditwarp.websocket.transporters']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'redditwarp',
    'version': '0.3.0',
    'description': 'The unofficial, type-complete Reddit API library for Python',
    'long_description': None,
    'author': 'Pyprohly',
    'author_email': 'pyprohly@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
