
from .token_obtainment_client_SYNC import TokenObtainmentClient as TokenObtainmentClient  # noqa: F401
from .token_revocation_client_SYNC import TokenRevocationClient as TokenRevocationClient  # noqa: F401
